/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: config.cc,v 1.5 2000/09/09 17:59:14 malekith Exp $ */

#include <v/utils.h>
#include <v/hash.h>
#include <v/re.h>

#include <stdio.h>

#include <unistd.h>

const char *v_default_config =
"# deafualt v config file
lynx-like-motion=1
lynx-like-select=0
";
void v_init_config();

static Hash <int> v_int_conf;

int v_get_int_conf(const String &r)
{
	v_init_config();
	return v_int_conf(r);
}

static Hash <String> v_string_conf;

String v_get_string_conf(const String &r)
{
	v_init_config();
	return v_string_conf(r);
}

String v_dump_config()
{
	String r, t;
	
	Block <String> k;
	
	k = v_int_conf.keys();
	for (int i=0; i<k.size(); i++) {
		t.setf("%s=%d\n",k[i].cstr(),v_int_conf(k[i]));
		r += t;
	}
	
	k = v_string_conf.keys();
	for (int i=0; i<k.size(); i++) {
		t.setf("%s:%s\n",k[i].cstr(),v_string_conf(k[i]).cstr());
		r += t;
	}
	
	return r;
}

void v_read_config(String fn)
{
	FILE *f;
	char line[1000];
	
	f = fopen(fn, "r");
	if (!f)
		throw XRecover("v_read_config(): can't open config file:" + fn);
	
	while (fgets(line, sizeof(line), f)) {
		if (line[strlen(line)-1] == '\n')
			line[strlen(line)-1] = 0;
		String l = line;

		if ((l *= "^[ \t]*#") || (l *= "^[ \t]*$"))
			continue;

		Regex re1("[ \t]*([^=]+)=([0-9]+)");
		Regex re2("[ \t]*([^:]+):(.*)");
		if (l *= re1)
			v_int_conf(re1.r(1)) = atoi(re1.r(2));
		else if (l *= re2)
			v_string_conf(re2.r(1)) = re2.r(2);
	}
	fclose(f);
}

void v_create_default_config(String fn)
{
	FILE *f = fopen(fn, "w");
	if (!f)
		throw XRecover("v_create_default_config(): cannot create: "+fn);
	fputs(v_default_config, f);
	fclose(f);
}

#define CONFIGFILE "vrc"

void v_init_config()
{
	String f = "/dev/null";
	static int x = 0;
	
	if (x)
		return;
	x = 1;
	
	if (getenv("CONFIG_DIR"))
		f.setf("%s/%s", getenv("CONFIG_DIR"), CONFIGFILE);
	else if (getenv("HOME"))
		f.setf("%s/.%s", getenv("HOME"), CONFIGFILE);
	
	if (access(f, R_OK) != 0)
		v_create_default_config(f);
	v_read_config(f);
	
	v_int_conf.undef = 0;
	v_string_conf.undef = "";
}
